// InteractiveMeshControl.cpp : implementation file
//
#include "stdafx.h"
#include "../resource.h"
#include "InteractiveMeshControl.h"


// InteractiveMeshControl dialog

IMPLEMENT_DYNAMIC(InteractiveMeshControl, CDialog)
InteractiveMeshControl::InteractiveMeshControl(CWnd* pParent /*=NULL*/)
	: CDialog(InteractiveMeshControl::IDD, pParent)
  , m_ShowNormal(FALSE)
  , m_ShowControlGrid(FALSE)
  , m_CustomMesh(NULL)
{
}

InteractiveMeshControl::~InteractiveMeshControl()
{
}

void InteractiveMeshControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Check(pDX, IDC_MESH_NORMAL, m_ShowNormal);
  DDX_Check(pDX, IDC_CONTROL_GRID, m_ShowControlGrid);
  DDX_Control(pDX, IDC_ROW_CV, m_RowSlider);
  DDX_Control(pDX, IDC_COL_CV, m_ColSlider);
  DDX_Control(pDX, IDC_POS_X,  m_PosXSlider);
  DDX_Control(pDX, IDC_POS_Y,  m_PosYSlider);
  DDX_Control(pDX, IDC_POS_Z,  m_PosZSlider);
}


BEGIN_MESSAGE_MAP(InteractiveMeshControl, CDialog)
  ON_BN_CLICKED(IDC_MESH_NORMAL, OnBnClickedMeshNormal)
  ON_BN_CLICKED(IDC_CONTROL_GRID, OnBnClickedControlGrid)
  ON_WM_HSCROLL()
END_MESSAGE_MAP()


void InteractiveMeshControl::SetCustomMesh(UWBD3D_PrimitiveMeshCustom *c) 
{ 
  m_CustomMesh = c; 
  InitializeControl( m_CustomMesh->Resolution() );
}

void 
InteractiveMeshControl::InitializeCVPos()
{
  vec3 p;
  if (m_CustomMesh == NULL)
    return;

  m_CustomMesh->SelectedVertexPosition(p);
#define DELTA 0.2f
#define DELTA_Y 2.0f
  m_PosXSlider.Initialize(p.x-DELTA, p.x+DELTA, p.x);
  m_PosYSlider.Initialize(p.y-DELTA_Y, p.y+DELTA_Y, p.y);
  m_PosZSlider.Initialize(p.z-DELTA, p.z+DELTA, p.z);
  UpdateData(FALSE);
}

void 
InteractiveMeshControl::InitializeControl(int res)
{
  if (m_CustomMesh != NULL) {
      m_ShowNormal = m_CustomMesh->DrawNormalIsEnabled();
      m_ShowControlGrid = m_CustomMesh->DrawControlGridIsEnabled();
  }
  
  m_RowSlider.Initialize(0.0f, res-1.0f, 0.0f);
  m_ColSlider.Initialize(0.0f, res-1.0f, 0.0f);
  m_CustomMesh->SelectVertex(0, 0);

  InitializeCVPos();

  UpdateData(FALSE);
}


// InteractiveMeshControl message handlers
BOOL InteractiveMeshControl::OnInitDialog()
{
  CDialog::OnInitDialog();

  m_ShowNormal = m_ShowControlGrid = false;
  m_CustomMesh = NULL;
  m_RowSlider.Initialize(0.0f, 1.0f, 0.0f);
  m_ColSlider.Initialize(0.0f, 1.0f, 0.0f);
  
  m_PosXSlider.Initialize(-1.0f, 1.0f, 0.0f);
  m_PosYSlider.Initialize(-1.0f, 1.0f, 0.0f);
  m_PosZSlider.Initialize(-1.0f, 1.0f, 0.0f);

  return TRUE;
}

void InteractiveMeshControl::OnBnClickedMeshNormal()
{
  UpdateData(TRUE);
  m_CustomMesh->SetDrawNormalTo( m_ShowNormal == TRUE );
}

void InteractiveMeshControl::OnBnClickedControlGrid()
{
    UpdateData(TRUE);
    m_CustomMesh->SetDrawControlGridTo( m_ShowControlGrid == TRUE);    
}


void InteractiveMeshControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  UWBMFC_SliderCtrlWithEcho *pSlider;

  int r, c;
  vec3 p;

  CDialog::OnHScroll(nSBCode, nPos, pScrollBar);

  if ( (pScrollBar == (CScrollBar*) & m_RowSlider) || (pScrollBar == (CScrollBar*) & m_ColSlider) ) {
	      r = int(m_RowSlider.GetSliderValue());
        c = int(m_ColSlider.GetSliderValue());
        
        m_CustomMesh->SelectVertex(r, c);
        InitializeCVPos();
        
        #define TestAndGetPtr( _sliderBar_)								        \
					if(pScrollBar == (CScrollBar*)&##_sliderBar_##)  {		\
							pSlider = &##_sliderBar_##;												\
					}
        TestAndGetPtr(m_RowSlider)
        TestAndGetPtr(m_ColSlider)
        
  } else {

      m_CustomMesh->SelectedVertexPosition(p);
      if ( (pScrollBar == (CScrollBar*) & m_PosXSlider) ) {
            p.x = m_PosXSlider.GetSliderValue();
            pSlider = &m_PosXSlider;
      } else {
            if (pScrollBar == (CScrollBar*) & m_PosYSlider)  {
                  p.y = m_PosYSlider.GetSliderValue();
                  pSlider = &m_PosYSlider;
            } else {
                  if (pScrollBar == (CScrollBar*) & m_PosZSlider) {
                      p.z = m_PosZSlider.GetSliderValue();
                      pSlider = &m_PosZSlider;
                  }
            }
      }
      m_CustomMesh->SetSelectedVertexPosition(p);
  }

  if (pSlider != NULL)
    CDialog::OnHScroll(nSBCode, nPos, pScrollBar);

}