#include "stdafx.h"
#include "../resource.h"
#include "PrimitiveControl.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

CPrimitiveControl::CPrimitiveControl( CWnd* pParent )
: CDialog(CPrimitiveControl::IDD, pParent)
, m_Primitive(NULL)
, m_Texture(FALSE)
, m_Visibility(TRUE)
, m_Shading(FALSE)
, m_Blending(FALSE)
, m_WireFrame(FALSE)
, m_BoundingVol(FALSE)
, m_Lighting(FALSE)
, m_SliderControl(eSliderControlObjColor)
{
}

void CPrimitiveControl::SetPrimitive(UWB_Primitive *p)
{
  m_Primitive = p;
  PrimitiveSetsControl(); 
}

void CPrimitiveControl::PrimitiveSetsControl()
{
  if( m_Primitive )
  {
    m_Texture = m_Primitive->IsTexturingEnabled() ? TRUE : FALSE;
    m_Blending = m_Primitive->IsBlendingEnabled() ? TRUE : FALSE;
    m_Visibility = m_Primitive->IsVisible() ? TRUE : FALSE;
    m_BoundingVol = m_Primitive->IsDrawBoundingVolume() ? TRUE : FALSE;
    m_Shading = (m_Primitive->GetShadeMode() == smGouraud) ? TRUE : FALSE;
    m_WireFrame = (m_Primitive->GetFillMode() == fmWireframe) ? TRUE : FALSE;
    m_Lighting = m_Primitive->IsLightingEnabled() ? TRUE : FALSE;

    m_TextureFileEcho = m_Primitive->GetTextureFileName();
    PrimitiveSetsColorSliders();
    UpdateData(FALSE);
  }

  if(IsWindow(m_hWnd))
  {
    if( m_Primitive )
      ShowWindow(SW_SHOW);
    else
      ShowWindow(SW_HIDE);
  }
}

void CPrimitiveControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_COLOR_R, m_ColorR);
  DDX_Control(pDX, IDC_COLOR_G, m_ColorG);
  DDX_Control(pDX, IDC_COLOR_B, m_ColorB);
  DDX_Control(pDX, IDC_COLOR_A, m_ColorA);
  DDX_Check(pDX, IDC_TEXTURE, 		m_Texture);
  DDX_Check(pDX, IDC_WIRE_FRAME, 	m_WireFrame);
  DDX_Check(pDX, IDC_VISIBILITY, 	m_Visibility);
  DDX_Check(pDX, IDC_BOUNDINGVOL, m_BoundingVol);
  DDX_Check(pDX, IDC_SHADING, 		m_Shading);
  DDX_Check(pDX, IDC_BLENDING, 		m_Blending);
  DDX_Check(pDX, IDC_LIGHTING,    m_Lighting);
  DDX_Text(pDX, IDC_TEXTURE_FILENAME, m_TextureFileEcho);
}

BOOL CPrimitiveControl::OnInitDialog()
{
  CDialog::OnInitDialog();

  m_ColorR.Initialize(0.0f, 1.0f, 1.0f);
  m_ColorG.Initialize(0.0f, 1.0f, 0.8f);
  m_ColorB.Initialize(0.0f, 1.0f, 0.8f);
  InitASliderToColor();
  
  PrimitiveSetsControl();
  CheckRadioButton(IDC_OBJECT_COLOR, IDC_SHININGNESS, IDC_OBJECT_COLOR);
  return TRUE;
}

BEGIN_MESSAGE_MAP(CPrimitiveControl, CDialog)
  ON_WM_HSCROLL()
  ON_WM_CTLCOLOR()
  ON_BN_CLICKED(IDC_TEXTURE,      OnBnClickedTexture)
  ON_BN_CLICKED(IDC_VISIBILITY,   OnBnClickedVisibility)
  ON_BN_CLICKED(IDC_SHADING,      OnBnClickedShading)
  ON_BN_CLICKED(IDC_BLENDING,     OnBnClickedBlending)
  ON_BN_CLICKED(IDC_WIRE_FRAME,   OnBnClickedWireFrame)
  ON_BN_CLICKED(IDC_BOUNDINGVOL,  OnBnClickedBoundingVol)
  ON_BN_CLICKED(IDC_SHADE_COLOR,  OnBnClickedShadeColor)
  ON_BN_CLICKED(IDC_OBJECT_COLOR, OnBnClickedObjectColor)
  ON_BN_CLICKED(IDC_BROWSE_TEXTUREFILE, OnBnClickedBrowseTexturefile)
  ON_BN_CLICKED(IDC_LIGHTING,     OnBnClickedLighting)
  ON_BN_CLICKED(IDC_DIFFUSE_COLOR, &CPrimitiveControl::OnBnClickedDiffuseColor)
  ON_BN_CLICKED(IDC_SPECULAR_COLOR, &CPrimitiveControl::OnBnClickedSpecularColor)
  ON_BN_CLICKED(IDC_SHININGNESS, &CPrimitiveControl::OnBnClickedShiningness)
END_MESSAGE_MAP()

HBRUSH CPrimitiveControl::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	
	// if the control is one of our color labels, change the text color
	if (pWnd->GetDlgCtrlID() == IDC_R_LABEL)
	{
		pDC->SetTextColor( RGB(255,0,0) );
		pDC->SetBkMode(TRANSPARENT);
	}
	else if (pWnd->GetDlgCtrlID() == IDC_G_LABEL)
	{
		pDC->SetTextColor( RGB(0,255,0) );
		pDC->SetBkMode(TRANSPARENT);
	}
	else if (pWnd->GetDlgCtrlID() == IDC_B_LABEL)
	{
		pDC->SetTextColor( RGB(0,0,255) );
		pDC->SetBkMode(TRANSPARENT);
  }

  return hbr;
}

void CPrimitiveControl::InitASliderToColor()
{
  m_ColorR.EnableWindow(TRUE);
  m_ColorG.EnableWindow(TRUE);
  m_ColorB.EnableWindow(TRUE);

  m_ColorA.Initialize(0.0f, 1.0f, 0.8f);
}

void CPrimitiveControl::InitASliderToN()
{ 
  m_ColorA.Initialize(0.0f, 200.0f, 10.0f);
  m_ColorR.EnableWindow(FALSE);
  m_ColorG.EnableWindow(FALSE);
  m_ColorB.EnableWindow(FALSE);  
}

void CPrimitiveControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  // For Update the SliderCtrlWithEcho's echoing
  if( pScrollBar == (CScrollBar*)&m_ColorR ||
      pScrollBar == (CScrollBar*)&m_ColorG ||
      pScrollBar == (CScrollBar*)&m_ColorB ||
      pScrollBar == (CScrollBar*)&m_ColorA )
  {
    ColorSlidersSetPrimitive();
  }
  
  CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CPrimitiveControl::OnBnClickedTexture()
{
  UpdateData(TRUE);
  if(m_Primitive)
    m_Primitive->EnableTexturing(m_Texture == TRUE);
}

void CPrimitiveControl::OnBnClickedVisibility()
{
  UpdateData(TRUE);
  if ( m_Primitive)
    m_Primitive->SetVisible( m_Visibility == TRUE );
}

void CPrimitiveControl::OnBnClickedShading()
{
  UpdateData(TRUE);
  if ( m_Primitive )
  {
    if (m_Shading)
      m_Primitive->SetShadeMode(smGouraud); 
    else
      m_Primitive->SetShadeMode(smFlat);
  }
}

void CPrimitiveControl::OnBnClickedBlending()
{
  UpdateData(TRUE);
  if ( m_Primitive )
    m_Primitive->EnableBlending( m_Blending == TRUE );
}

void CPrimitiveControl::OnBnClickedLighting()
{
  UpdateData(TRUE);
  if (m_Primitive) {
    m_Primitive->EnableLighting( m_Lighting == TRUE );
    PrimitiveSetsColorSliders();
  }
}

void CPrimitiveControl::OnBnClickedWireFrame()
{
  UpdateData(TRUE);
  if( m_Primitive )
  {
    if (m_WireFrame )
      m_Primitive->SetFillMode(fmWireframe);
    else
      m_Primitive->SetFillMode(fmSolid);
  }
}

void CPrimitiveControl::OnBnClickedBoundingVol()
{
  UpdateData(TRUE);
  if( m_Primitive )
    m_Primitive->SetDrawBoundingVolume( m_BoundingVol == TRUE );
}

void CPrimitiveControl::OnBnClickedBrowseTexturefile()
{
  CFileDialog file_dlg( TRUE );
  if( IDOK == file_dlg.DoModal() )
  {
    CString path = file_dlg.GetPathName();
    m_TextureFileEcho = file_dlg.GetFileName();
    m_Primitive->SetTextureFileName( path );
    UpdateData(FALSE);
  }
}
void CPrimitiveControl::PrimitiveSetsColorSliders()
{
  if (NULL == m_Primitive)
    return;


  UWB_Material m = m_Primitive->GetMaterial();

  if (m_SliderControl == eSliderControlShiningness) {
        float n = m.GetPower();
        m_ColorA.SetSliderValue(n);
  } else {


      UWB_Color c(0, 0, 0, 0);

      switch (m_SliderControl)
      {
        case eSliderControlObjColor:
          if (m_Lighting)
              m.GetEmissive(c);
          else
              c = m_Primitive->GetFlatColor();
          break;

        case eSliderControlShadeColor:
          if (m_Lighting)
            m.GetAmbient(c);
          else 
            c = m_Primitive->GetShadingColor();
          break;

        case eSliderControlDiffuseColor:
            m.GetDiffuse(c);
            break;
            
          case eSliderControlSpecularColor:
            m.GetSpecular(c);
            break;
      }
      m_ColorR.SetSliderValue(c.PercentRed());
      m_ColorG.SetSliderValue(c.PercentGreen());
      m_ColorB.SetSliderValue(c.PercentBlue());
      m_ColorA.SetSliderValue(c.PercentAlpha());
  }
  UpdateData(FALSE);
}

void CPrimitiveControl::ColorSlidersSetPrimitive()
{
  if (m_Primitive == NULL)
      return;

  UWB_Color c(m_ColorR.GetSliderValue(), m_ColorG.GetSliderValue(),
              m_ColorB.GetSliderValue(), m_ColorA.GetSliderValue());

  UWB_Material m = m_Primitive->GetMaterial();
  switch (m_SliderControl)
  {
      case eSliderControlObjColor:
        if (m_Lighting)
            m.SetEmissive(c);
        else
            m_Primitive->SetFlatColor(c);
        break;

      case eSliderControlShadeColor:
        if (m_Lighting)
            m.SetAmbient(c);
        else
            m_Primitive->SetShadingColor(c);
        break;
    
      case eSliderControlDiffuseColor:
        m.SetDiffuse(c);
        break;
        
      case eSliderControlSpecularColor:
        m.SetSpecular(c);
        break;

      case eSliderControlShiningness:
        m.SetPower(m_ColorA.GetSliderValue());
        break;
  }
  m_Primitive->SetMaterial(m);
}

void CPrimitiveControl::OnBnClickedObjectColor()
{
  if (m_SliderControl == eSliderControlShiningness)
      InitASliderToColor();
  m_SliderControl = eSliderControlObjColor;
  PrimitiveSetsColorSliders();
}

void CPrimitiveControl::OnBnClickedShadeColor()
{
  if (m_SliderControl == eSliderControlShiningness)
      InitASliderToColor();
  m_SliderControl = eSliderControlShadeColor;
  PrimitiveSetsColorSliders();
}

void CPrimitiveControl::OnBnClickedDiffuseColor()
{
  if (m_SliderControl == eSliderControlShiningness)
      InitASliderToColor();
  m_SliderControl = eSliderControlDiffuseColor;
  PrimitiveSetsColorSliders();
}

void CPrimitiveControl::OnBnClickedSpecularColor()
{
  if (m_SliderControl == eSliderControlShiningness)
      InitASliderToColor();
  m_SliderControl = eSliderControlSpecularColor;
  PrimitiveSetsColorSliders();
}

void CPrimitiveControl::OnBnClickedShiningness()
{
  InitASliderToN();
  m_SliderControl = eSliderControlShiningness;
  PrimitiveSetsColorSliders();
}