#include "stdafx.h"
#include "XformInfoControl.h"
#include "../TutorialApp.h"

// DEBUG_NEW macro allows MFC applications to determine memory leak locations in debug builds
#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

CXformInfoControl::CXformInfoControl(CWnd* pParent)
: CDialog(CXformInfoControl::IDD, pParent)
, m_TranslateMin(-10.0f)
, m_TranslateMax(10.0f)
, m_ScaleMin(0.1f)
, m_ScaleMax(10.0f)
, m_RotateMin(-180.0f)
, m_RotateMax(180.0f)
, m_bShowPivot(FALSE)
, m_SceneNode(NULL)
, m_rx(0.0f)
, m_ry(0.0f)
, m_rz(0.0f)
{
}

void CXformInfoControl::SetSliderRange(float tmin, float tmax, float smin, float smax, float rmin, float rmax)
{
  m_TranslateMin = tmin;
  m_TranslateMax = tmax;
  m_ScaleMin = smin;
  m_ScaleMax = smax;
  m_RotateMin = rmin;
  m_RotateMax = rmax;
  InitializeSliders();
}

void CXformInfoControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_XR_SLIDER, m_XSlider);
  DDX_Control(pDX, IDC_Y_SLIDER, m_YSlider);
  DDX_Control(pDX, IDC_Z_SLIDER, m_ZSlider);
  DDX_Check(pDX, IDC_CHECK_SHOW_PIVOT, m_bShowPivot);
}

BEGIN_MESSAGE_MAP(CXformInfoControl, CDialog)
  ON_BN_CLICKED(IDC_TRANSLATE, OnRadioClickedXformControl)
  ON_BN_CLICKED(IDC_SCALE, OnRadioClickedXformControl)
  ON_BN_CLICKED(IDC_ROTATE, OnRadioClickedXformControl)
  ON_BN_CLICKED(IDC_PIVOT, OnRadioClickedXformControl)
  ON_BN_CLICKED(IDC_RESET, OnBnClickedReset)
  ON_BN_CLICKED(IDC_CHECK_SHOW_PIVOT, OnBnClickedCheckShowPivot)
  ON_WM_HSCROLL()
END_MESSAGE_MAP()

void CXformInfoControl::SetSceneNode(UWB_SceneNode *node) 
{  
  m_SceneNode = node; 
  InitializeSliders();
}

void CXformInfoControl::InitializeSliders()
{
  if (NULL==m_SceneNode)
    return;

  if( IsWindow(m_hWnd) )
  {
    int current_control = GetCheckedRadioButton(IDC_TRANSLATE, IDC_PIVOT);
    switch(current_control)
    {
    case IDC_TRANSLATE:
      m_XSlider.Initialize( m_TranslateMin, m_TranslateMax, m_SceneNode->GetXformInfo().GetTranslation().x );
      m_YSlider.Initialize( m_TranslateMin, m_TranslateMax, m_SceneNode->GetXformInfo().GetTranslation().y );
      m_ZSlider.Initialize( m_TranslateMin, m_TranslateMax, m_SceneNode->GetXformInfo().GetTranslation().z );
      break;
    case IDC_SCALE:
      m_XSlider.Initialize( m_ScaleMin, m_ScaleMax, m_SceneNode->GetXformInfo().GetScale().x );
      m_YSlider.Initialize( m_ScaleMin, m_ScaleMax, m_SceneNode->GetXformInfo().GetScale().y );
      m_ZSlider.Initialize( m_ScaleMin, m_ScaleMax, m_SceneNode->GetXformInfo().GetScale().z );
      break;
    case IDC_ROTATE:
      {
      m_rx = m_ry = m_rz = 0.0f;
      m_XSlider.Initialize( m_RotateMin, m_RotateMax, m_rx );
      m_YSlider.Initialize( m_RotateMin, m_RotateMax, m_ry );
      m_ZSlider.Initialize( m_RotateMin, m_RotateMax, m_rz );
      }
      break;
    case IDC_PIVOT:
      m_XSlider.Initialize( m_TranslateMin, m_TranslateMax, m_SceneNode->GetXformInfo().GetPivot().x );
      m_YSlider.Initialize( m_TranslateMin, m_TranslateMax, m_SceneNode->GetXformInfo().GetPivot().y );
      m_ZSlider.Initialize( m_TranslateMin, m_TranslateMax, m_SceneNode->GetXformInfo().GetPivot().z );
      break;
    }
    m_bShowPivot = m_SceneNode->IsPivotVisible();
    UpdateData(FALSE);
  }
}

BOOL CXformInfoControl::OnInitDialog()
{
  CDialog::OnInitDialog();
  m_SceneNode = NULL;
  CheckRadioButton(IDC_TRANSLATE, IDC_PIVOT, IDC_TRANSLATE);
  InitializeSliders();
  return TRUE;
}

// CXformInfoControl message handlers

void CXformInfoControl::OnRadioClickedXformControl()
{
  InitializeSliders();
}

void CXformInfoControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  if (NULL==m_SceneNode)
    return;

  // get slider values
  float x = m_XSlider.GetSliderValue();
  float y = m_YSlider.GetSliderValue();
  float z = m_ZSlider.GetSliderValue();
  
  UWB_XformInfo xform = m_SceneNode->GetXformInfo();

  int current_control = GetCheckedRadioButton(IDC_TRANSLATE, IDC_PIVOT);
  switch(current_control)
  {
  case IDC_TRANSLATE:
    xform.SetTranslation( vec3(x,y,z) );
    break;
  case IDC_SCALE:
    xform.SetScale( vec3(x,y,z) );
    break;
  case IDC_ROTATE:
    {
      float dx = x - m_rx;
      float dy = y - m_ry;
      float dz = z - m_rz;
      m_rx = x;
      m_ry = y;
      m_rz = z;
      xform.UpdateRotationXByDegree(dx);
      xform.UpdateRotationYByDegree(dy);
      xform.UpdateRotationZByDegree(dz);
    }
    break;
  case IDC_PIVOT:
    xform.SetPivot( vec3(x,y,z) );
    break;
  }
  m_SceneNode->SetXformInfo(xform);
  CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}
void CXformInfoControl::OnBnClickedReset()
{
  UWB_XformInfo reset;
  m_SceneNode->SetXformInfo( reset );
  InitializeSliders();
}

void CXformInfoControl::OnBnClickedCheckShowPivot()
{
  UpdateData(TRUE);
  if (NULL != m_SceneNode)
    m_SceneNode->SetPivotVisible(m_bShowPivot==TRUE);
}