// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once
#include "../Resource.h"

/// Child dialog that contains a set of transformation controls
/// for setting matrix transformations used during primitive drawing
class CXformInfoControl : public CDialog
{
public:
  CXformInfoControl(CWnd* pParent = NULL);   // standard constructor

  /// The IDD enum is a common technique in MFC to associate a dialog with
  /// a resource that you edit in the dialog editor
  enum { IDD = IDD_XFORMCONTROL_DIALOG };

  /// Set the ranges for the translate, scale, and rotate slider controls
  /// \param tmin, tmax Range for the translation slider
  /// \param smin, smax Range for the scale slider
  /// \param rmin, rmax Range for the rotation slider
  void SetSliderRange(float tmin, float tmax, float smin, float smax, float rmin, float rmax);
  void SetSceneNode(UWB_SceneNode *node);

protected:
  /// The virtual DoDataExchange is an MFC method for synchronizing values in this
  /// class with their corresponding controls on the dialog.
  /// See MSDN documentation for more information
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

  /// Called immediately after a dialog is created, but before it is displayed.
  /// This is where all of the control initialization usually occurs. This is
  /// also where the graphics window is set up
  virtual BOOL OnInitDialog();

  /// A message map is an MFC macro for mapping window's events (paint, size, mouse,...) that
  /// occur on a window to functions in this class.
  /// The functions in this class that begin with afx_msg are "handlers" for window's messages
  DECLARE_MESSAGE_MAP()

  afx_msg void OnBnClickedCheckShowPivot();

  /// Called when the radio state for the current type of transformation the
  /// sliders control changes
  afx_msg void OnRadioClickedXformControl();

  ///
  /// Called when the slider states change through user input.
  afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);

  ///
  /// Called when the reset button is clicked
  afx_msg void OnBnClickedReset();

  ///
  /// Set up the slider controls state to match the saved variable values in this class
  void InitializeSliders();

  UWBMFC_SliderCtrlWithEcho m_XSlider;
  UWBMFC_SliderCtrlWithEcho m_YSlider;
  UWBMFC_SliderCtrlWithEcho m_ZSlider;

  // The XformInfo this control works on.
  float m_rx, m_ry, m_rz;
  BOOL m_bShowPivot;
  UWB_SceneNode *m_SceneNode;
  float m_TranslateMin, m_TranslateMax, m_ScaleMin, m_ScaleMax, m_RotateMin, m_RotateMax;
};
