#include "StdAfx.h"
#include "Model.h"

CModel::CModel()
: m_root_node( L"root node" )
, m_PointLight(1)
, m_DirectionalLight(2)
, m_SpotLight(3)
, m_MoveTiger(false)
, m_TigerCam(NULL)
{
  m_pTigerNode = new UWB_SceneNode( L"tiger" );
  UWBD3D_PrimitiveMesh *tigerMesh = new UWBD3D_PrimitiveMesh( L"tiger.x" );
  tigerMesh->EnableTexturing(true);
  m_pTigerNode->SetPrimitive(tigerMesh);
  UWB_XformInfo txf;
  txf.SetTranslation(vec3(-3, 2, 4));
  m_pTigerNode->SetXformInfo(txf);
  m_root_node.InsertChildNode( m_pTigerNode );
  
  UWB_SceneNode* pFloorNode = new UWB_SceneNode(L"CustomMesh");
  m_CustomMesh = new UWBD3D_PrimitiveMeshCustom(30, L"HiResMesh.x");
  pFloorNode->SetPrimitive(m_CustomMesh);
  UWB_XformInfo floorXf;
  floorXf.SetScale(vec3(15.0f, 15.0f, 15.0f));
  pFloorNode->SetXformInfo(floorXf);
  m_root_node.InsertChildNode( pFloorNode );
  
  m_pShusuiNode = new UWB_SceneNode(L"shusui");
  UWBD3D_PrimitiveMesh *pCraft = new UWBD3D_PrimitiveMesh(L"shusui.x");
  UWB_XformInfo sxf;
  sxf.SetTranslation(vec3(3, 5, -3));
  m_pShusuiNode->SetXformInfo(sxf);
  m_pShusuiNode->SetPrimitive(pCraft);
  m_root_node.InsertChildNode(m_pShusuiNode);

  m_pTeapotNode = new UWB_SceneNode(L"teapot");
  UWBD3D_PrimitiveMesh *pTeapot = new UWBD3D_PrimitiveMesh(L"teapot.x");
  UWB_XformInfo pxf;
  pxf.SetTranslation(vec3(-3, 1, -4));
  m_pTeapotNode->SetXformInfo(pxf);
  m_pTeapotNode->SetPrimitive(pTeapot);
  m_root_node.InsertChildNode(m_pTeapotNode);

  UWB_SceneNode* pAxisNode = new UWB_SceneNode( L"axis frame" );
  UWBD3D_PrimitiveMeshAxis* pMeshAxis = new UWBD3D_PrimitiveMeshAxis(4.0f);
  pAxisNode->SetPrimitive( pMeshAxis );
  m_root_node.InsertChildNode(pAxisNode);

  // Lighting condition ...
  m_PointLight.SwitchOnLight();
  m_PointLight.SetPosition(2.0f, 12.0f, 2.0f);
  m_PointLight.SetDiffuse(0.2f);

  m_DirectionalLight.SwitchOnLight();
  m_DirectionalLight.SetPosition(0.0f, 12.0f, 0.0f);
  m_DirectionalLight.SetDiffuse(0.2f, 0.1f, 0.1f);

  m_SpotLight.SwitchOnLight();
  m_SpotLight.SetPosition(-2.0f, 12.0f, 2.0f);
  m_SpotLight.SetDiffuse(0.2f, 0.2f, 0.5f);

  m_ShowTigerCam = new UWB_SceneNode(L"show tiger");
  UWBD3D_PrimitiveMeshAxis *a = new UWBD3D_PrimitiveMeshAxis(0.8f);
  m_ShowTigerCam->SetPrimitive(a);
}

void CModel::DrawModel()
{
    UWBD3D_DrawHelper draw_helper;  

    m_PointLight.SetLightToDevice();
		m_DirectionalLight.SetLightToDevice();
		m_SpotLight.SetLightToDevice();
				 
		m_DirectionalLight.DrawLight(draw_helper);
		m_SpotLight.DrawLight(draw_helper);
		m_PointLight.DrawLight(draw_helper);

  
    m_root_node.Draw( draw_helper.GetLOD(), draw_helper );

    UWB_Camera *c = (NULL!=m_TigerCam)? m_TigerCam : m_FollowCam;
    if (NULL != c) 
    {
        UWB_XformInfo txf = m_pTigerNode->GetXformInfo();
        vec3 p = c->GetCameraPosition();
        vec3 at = c->GetCameraLookAt();
        UWB_XformInfo sxf = m_ShowTigerCam->GetXformInfo();
        sxf.SetRotationQuat( txf.GetRotationQuat() );
        sxf.SetTranslation(p);
        m_ShowTigerCam->SetXformInfo(sxf);
        m_ShowTigerCam->Draw( draw_helper.GetLOD(), draw_helper);

        UWB_PrimitiveLine ln;
        ln.SetStartPoint(p.x, p.y, p.z);
        ln.SetEndPoint(at.x, at.y, at.z);
        ln.SetShadeMode(smGouraud);
        ln.SetShadingColor(UWB_Color(1.0f, 1.0f, 1.0f, 0.0f));
        ln.SetFlatColor(UWB_Color(1.0f, 0.0f, 0.0f, 0.0f));
        ln.EnableLighting(false);
        ln.Draw(draw_helper.GetLOD(), draw_helper);
    }
}

UWB_SceneNode* CModel::GetRootNode()
{
  return &m_root_node;
}