// University of Washington Bothell Graphics Library
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
///////////////////////////////////////////////////////////////////////////////////
#pragma once


/// The model separates the geometry and simulation of geometry interactions
/// from the user interface. Separation of the model from the user interface
/// generally leads to easier to maintain applications
class CModel
{
public:
  CModel();

  ///
  /// Draw all of the geometry that is part of this model.
  void DrawModel();

  ///
  /// Get the root node of the complete scene
  UWB_SceneNode* GetRootNode();
  void SetActiveSceneNode(UWB_SceneNode *n) {}
  UWBD3D_PrimitiveMeshCustom *GetCustomMesh() { return m_CustomMesh; }

  void SetMoveTiger(bool b) { m_MoveTiger = b; }
  void SetTigerCam(UWB_Camera *c) { m_TigerCam = c; }
  void SetFollowCam(UWB_Camera *c) { m_FollowCam = c; }

  UWB_SceneNode *GetShusui() { return m_pShusuiNode; }
  UWB_SceneNode *GetTeapot() { return m_pTeapotNode; }
  UWB_SceneNode *GetTiger() { return m_pTigerNode; }

  void UpdateSimulation();
  

  UWBD3D_Light*		GetSpotLight() { return &m_SpotLight; }
	UWBD3D_Light*		GetDirectionalLight() { return &m_DirectionalLight; }
	UWBD3D_Light*		GetPointLight() { return &m_PointLight; }

private:

  void ComputeTigerPosition();
  void ComputeTigerCam();
  void ComputeFollowCam();


  UWB_Clock     m_StopWatch;

  UWB_SceneNode m_root_node;

  UWBD3D_PrimitiveMeshCustom *m_CustomMesh;
  
  // The lights 
  UWBD3D_PointLight        m_PointLight;
  UWBD3D_DirectionalLight  m_DirectionalLight;
  UWBD3D_SpotLight         m_SpotLight;


  UWB_SceneNode   *m_pTigerNode;
  UWB_SceneNode   *m_pShusuiNode;
  UWB_SceneNode   *m_pTeapotNode;

  bool            m_MoveTiger;

  UWB_Camera     *m_TigerCam;
  UWB_Camera     *m_FollowCam;
  UWB_SceneNode  *m_ShowTigerCam;
};