#include "uwbgl_D3DDirectionalLight1.h"
#include "uwbgl_D3DPrimitiveMeshArrow4.h"
#include "uwbgl_XformInfo6.h"

UWBD3D_DirectionalLight::UWBD3D_DirectionalLight(int id) : UWBD3D_Light(id)
{
  SetToDirectionalLight();
}

UWBD3D_DirectionalLight::~UWBD3D_DirectionalLight()
{
}

void UWBD3D_DirectionalLight::DrawLightGeometry(UWB_DrawHelper& draw_helper)
{
  // We will draw directional light with the arrow. 
  // our arrows initially points in the +ve x-direction
  // we need a rotation that will rotate y-axis toward m_d3dLight.Direction
  vec3 d, initDir(1.0f, 0.0f, 0.0f);
  GetDirection(d);
  normalize(d);
  float cosTheta = dot(d, initDir);
  float theta = acosf( cosTheta );
  
  vec3 rAxis(initDir);
  if (fabs(cosTheta) > 0.999f) {
        // almost on the same line ...
        if (cosTheta < 0.0f) {
              // pointing in the opposite direciont
              theta = UWB_PI;
        } else {
              // point in the same direction, we are done
              theta = 0.0f;
        }
  } else {
        rAxis = cross(initDir, d);
  }
  
  quat q = quatFromAA(rAxis, UWB_ToDegrees(theta));

  UWB_Color c;
  GetDiffuse(c);
  UWBD3D_PrimitiveMeshArrow a;
  a.SetFlatColor(c);

  UWB_XformInfo xf;
  xf.ToIdentity();
  xf.SetTranslation(vec3(m_d3dLight.Position.x, m_d3dLight.Position.y, m_d3dLight.Position.z));
  xf.SetScale(vec3(1.2f, 1.0f, 1.2f));
  xf.SetRotationQuat(q);
  
  draw_helper.PushModelTransform();
      xf.SetUpModelStack(draw_helper);
      a.Draw(draw_helper.GetLOD(), draw_helper);
  draw_helper.PopModelTransform();
}