#include "uwbgl_PrimitiveCircle6.h"

UWB_PrimitiveCircle::UWB_PrimitiveCircle()
: m_center(0,0,0)
, m_radius(0)
{
  // always keep the bounds in synch with the circle geometry
  m_bounds.SetCorners( vec3(m_center.x - m_radius, m_center.y - m_radius, 0),
                       vec3(m_center.x + m_radius, m_center.y + m_radius, 0));
}

void UWB_PrimitiveCircle::SetupDrawAttributes(UWB_DrawHelper& draw_helper)
{
  __super::SetupDrawAttributes(draw_helper);
  draw_helper.SetColor2(m_ShadingColor);
  draw_helper.SetShadeMode(m_ShadeMode);
  draw_helper.SetFillMode(m_FillMode);
}

void UWB_PrimitiveCircle::DrawPrimitive( eLevelOfDetail lod, UWB_DrawHelper& draw_helper )
{
  eLevelOfDetail oldlod = draw_helper.GetLOD();
  draw_helper.SetLOD(lod);
  draw_helper.DrawCircle(m_center, m_radius);
  draw_helper.SetLOD(oldlod);
}

void UWB_PrimitiveCircle::MouseDownVertex(int vertexID, float x, float y)
{
  if(vertexID == 0)
  {
    // First vertex creates a new circle by setting the center point to x,y and
    // the radius to zero
    SetCenter(x,y);
    SetRadius(0);
  }
  else
  {
    // Second vertex defines the circle's radius by using the second vertex as a point
    // on the edge of the circle
    float dx = x - m_center.x;
    float dy = y - m_center.y;
    float radius = sqrtf(dx*dx + dy*dy);
    SetRadius(radius);
  }
}

void UWB_PrimitiveCircle::MoveTo(float x, float y)
{
  SetCenter(x,y);
}

vec3 UWB_PrimitiveCircle::GetLocation() const
{
    return GetCenter();
}

void UWB_PrimitiveCircle::SetCenter(float x, float y, float z)
{
  m_center = vec3(x,y,z);
  // always keep the bounds in synch with the circle geometry
  m_bounds.SetCorners( vec3(m_center.x - m_radius, m_center.y - m_radius, 0),
                       vec3(m_center.x + m_radius, m_center.y + m_radius, 0));
}

void UWB_PrimitiveCircle::SetCenter(vec3 center)
{
  m_center = center;
  // always keep the bounds in synch with the circle geometry
  m_bounds.SetCorners( vec3(m_center.x - m_radius, m_center.y - m_radius, 0),
                       vec3(m_center.x + m_radius, m_center.y + m_radius, 0));
}

vec3 UWB_PrimitiveCircle::GetCenter() const
{
  return m_center;
}

void UWB_PrimitiveCircle::SetRadius(float radius)
{
  m_radius = radius;
  // always keep the bounds in synch with the circle geometry
  m_bounds.SetCorners( vec3(m_center.x - m_radius, m_center.y - m_radius, 0),
                       vec3(m_center.x + m_radius, m_center.y + m_radius, 0));
}

float UWB_PrimitiveCircle::GetRadius() const
{
  return m_radius;
}

void UWB_PrimitiveCircle::Update(float elapsed_seconds)
{
  // just moving the center point will move the entire circle
  // current position is defined by previous position + velocity * time
  vec3 adjust_vec = m_velocity * elapsed_seconds;
  m_center += adjust_vec;
  // always keep the bounds in synch with the circle geometry
  m_bounds.SetCorners( vec3(m_center.x - m_radius, m_center.y - m_radius, 0),
                       vec3(m_center.x + m_radius, m_center.y + m_radius, 0));
}

const UWB_BoundingVolume* UWB_PrimitiveCircle::GetBoundingVolume(eLevelOfDetail lod) const
{
  return &m_bounds;
}
